/*

BBC Radio widget by Hawkman. http://www.phantomgorilla.com/
All suggestions and bug reports to hawkman@phantomgorilla.com please

*******************************************************************************

LICENCE

Free to use, free to look at and learn from, free to small chunks of code from for use in your own projects (eg. reproduction of a couple of functions). You are also free to modify it as you see fit, for your own personal use only.

However, you may NOT redistribute this widget, or a port or modified version of it, without obtaining permission from the author (via email, hawkman@phantomgorilla.com) and including credit to him in the finished project (a small notice in the widget's javascript/html files is all that's necessary; nothing visible on the widget).

Small scale distribution of this widget or a modified version of it, such as that between family members or friends, is specifically allowed despite these restrictions.

This software is provided as-is, with no guarantee. The author can accept no responsibility for any consequences of its use.

*******************************************************************************

ABOUT THE WIDGET

- Basic Overview

This widget uses Real Player for the audio. Preferences are written upon the selection of an item in a pop-up menu, then this is compared to a list of stations to find the stream url. A function then writes html to the front of the widget to embed the realplayer plugin with the correct url, and changes the image source to reflect which station is playing.

- Credits

Basic Javascript routines taken out of Apple's sample Goodbye World widget. A small section of crash-preventing code is taken from Duncan Ponting's radio widget (http://www.bbc.co.uk/radio/prototypes/listenlive/) at his suggestion - cheers. Also taken from that widget is the idea to crawl the broswer plugins to find RealPlayer. Version string .plist lookup taken from Joshua Emmons' blank foo.wdgt (http://www.skia.net/). Study of his example AppleAnimator use was also extremely helpful. Some button-like routines adapted from Apple's GenericButton class.

Additional skills were obtained by studying Apple's sample files and the Mac OS X bundled widgets, however no actual code was used from these sources other than that mentioned already.

******************************************************************************/



/*******************************/
// GLOBAL VARIABLES & SETTINGS
/*******************************/

var debug = false;
if (!window.widget) { debug = true; }

// the kind of player to use
var useType;

// Global variable for the status animation
var statusAnimator = null;
var statusTimeout = null;
// and one for the drawer animation
var drawerAnimator = null;

var volumeCounter;

// sets up a variable to store the XML from checkForUpdate(), also to let
// us know if a request is currently in progress
var externalXmlRequest = null;

// variable to store the current version number from Info.plist; only set on
// load, but referred to on multiple occasions
var currentVersion;

// holds the version number of the latest version available; initially taken
// from preferences (if there), it's updated every day by checkForUpdate()
var latestVersion = new Object();

var updateUrl = (window.widget ? widget.preferenceForKey("UpdateUrl") : null);
// get the last time we checked for an update
var then = (window.widget ? widget.preferenceForKey("lastupdatecheck") : 0);
var lastShowFlashWarning = 0;

// all involved with the button handlers such as theMouseDownHandler //
// this one stores whether the mouse button is down
var theCurrentMouseDownButton = null;
// the button which was clicked
var wasClickedElement = null;
// the element to swap images into
var isElementToChange = null;
// the starting position of the left side of the volume knob
var knobStartLeft = null;
// the cursor's starting x-coordinate; used with the slider
var cursorStartX = null;
// left boundary of the slider
var leftBound = 76;
// right boundary of the slider
var rightBound = 108;

// to remember the selection before showing the rear
var preferenceBefore = null;

// used to temporarily store event settings if we use the showStatus() function,
// as we can't pass these directly from it to hideStatus() via setTimeout().
var globalStatusEvent;

// the xmlhttprequest object we use later
var xmlhttp;
var testInProg = null;

// flash flag
var UseFlash = false;
var flashTimer = new Object();
flashTimer.seconds = 0;
flashTimer.trigger;

var noHTML5 = false;

// flags for hidden plist keys
var disableTests = false;

var justShownRestartMessage = false;

// sets our onshow function to execute at the right times
if (window.widget) {
	widget.onshow = onShow;
	widget.onhide = onHide;
    widget.onremove = onRemove;
}

// stations listed in stations.js
var currentStation = stationList.radio1;


myScrollwheel = function(event)
{
    var area = document.getElementById("scrollArea").object;
	if (event.wheelDeltaY) {
        var deltaScroll = event.wheelDeltaY / 180 * area.singlepressScrollPixels;
    } else if (event.wheelDeltaX) {
        var deltaScroll = 0;
    } else {
        var deltaScroll = event.wheelDelta / 180 * area.singlepressScrollPixels;
    }
    // make sure we scroll, however little we've moved
    if (deltaScroll<1 && deltaScroll>0) deltaScroll=1;
    if (deltaScroll>-1 && deltaScroll<0) deltaScroll=-1;

	area.verticalScrollTo(area.content.scrollTop - deltaScroll);

	event.stopPropagation();
	event.preventDefault();
}


/*****************************************************/
// SETUP FUNCTIONS, SAVING AND RETRIEVING PREFERENCES
/*****************************************************/


// onLoad() is run when the body loads.  It checks to see if there is a
// preference for this widget and if so, applies the preference to the widget.
function onLoad()
{

    // draw the info button, using AppleClasses
    new AppleInfoButton (document.getElementById('info'), 
                     document.getElementById('front'), 
                     'black', 'black', showPrefs);
    // a clone of Apple's info button with minor modifications
    new drawerButton (document.getElementById('drawerbutton'), 
                     document.getElementById('front'), 
                     'black', doDrawerAction);

    if(!window.widget) { // create a box for alerts if we're not in dashboard
    
        var iDiv = document.createElement('div');
        iDiv.id = 'alerts';
        document.getElementById('bd').appendChild(iDiv);
        
        window.alert=function() { 
            var al = document.getElementById("alerts");
            var t = Math.round(+new Date()/1000);
            al.innerHTML += "<p class=\"alert\">"+t+": "+arguments[0]+"</p>";
            al.scrollTop = al.scrollHeight;
        } 
    }

    latestVersion.release = (window.widget ? widget.preferenceForKey("latestversion") : "");
    latestVersion.release = splitVersion(latestVersion.release);
    latestVersion.betaRelease = (window.widget ? widget.preferenceForKey("LatestBeta") : "");
    latestVersion.betaRelease = splitVersion(latestVersion.betaRelease);

    if(window.widget) { // always check to make sure that you are running in Dashboard

        // do some upgrade-related cleanup
        var lastUsedVersion = widget.preferenceForKey("last_used_version");
        if (!lastUsedVersion || lastUsedVersion.length <=0) {
            var lastUsedVersion = widget.preferenceForKey("LastUsedVersion");
        }
        lastUsedVersion = splitVersion(lastUsedVersion);
        if (widget.preferenceForKey("WaitingForRestart")) {
            widget.setPreferenceForKey(null,"WaitingForRestart");
            //if (lastUsedVersion==currentVersion) showStatus(null,'Thanks!<br>BBC Radio Widget has<br>now been successfully<br>upgraded to v'+currentVersion+'.',8);
        }
    }
    // calls fillVersion() to fill in the version number on the rear
    fillVersion();

   if(window.widget) { // always check to make sure that you are running in Dashboard

        // upgrade if needed, else load current prefs
        if (lastUsedVersion.major==0 || checkIfVersionHigher(lastUsedVersion, currentVersion)) {
            doUpgrade(lastUsedVersion);
        }
        // loads the current station from preferences and looks it up
        // first tries a key for this instance
        var temp_currentStation = widget.preferenceForKey(makeKey("CurrentStation"));
        // if there's nothing for this instance
        if (!temp_currentStation || temp_currentStation.length <= 0) {
            // loads default key for new instances
            temp_currentStation = widget.preferenceForKey("CurrentStation");
        }
        if (debug) alert("Versions match. No upgrade needed. Loading current station: "+temp_currentStation);
        // check it makes sense and load the station's info
        if (temp_currentStation && temp_currentStation.length > 0) {
            if (temp_currentStation in stationList) {
                currentStation = stationList[temp_currentStation];
            }
            // otherwise default kept; either way,
            // save for this instance and as default for new instances
            widget.setPreferenceForKey(currentStation.nickname+(currentStation.extra ? currentStation.extra : ""),"CurrentStation");
            widget.setPreferenceForKey(currentStation.nickname+(currentStation.extra ? currentStation.extra : ""),makeKey("CurrentStation"));
        }
    }
    // rebuilds the station selection menu
    buildStationSelect();
    // gets and shows the correct station image
    changeImage();
    // changes the image on the schedule drawer button
    updateDrawerButton();

    // sets the volume slider to the correct position
    restoreVolume();
    
    // draw the Done button for the back - again, AppleClasses
    new AppleGlassButton(document.getElementById('done'), 'Done', hidePrefs);
    //new AppleGlassButton(document.getElementById('done2'), 'Done', hidePrefs2);
    
    // make schedule scroll less dramatically fast
    var area = document.getElementById("scrollArea").object;
    area.content.removeEventListener("mousewheel", area._mousewheelScrollHandler, true);
    area.content.addEventListener("mousewheel", myScrollwheel, true);

    // lets us disable connection/plugin tests
    checkForTestDisable();

    // refreshes the schedule
    refreshSchedule();

    checkForUpdate();
}


// delete temporary preferences, stop timers
function onRemove() {
    widget.setPreferenceForKey(null,makeKey("CurrentStation"));
    widget.setPreferenceForKey(null,makeKey("Volume"));
    clearTimeout(volumeCounter);
    clearTimeout(statusTimeout);
    clearTimeout(testInProg);
    clearInterval(flashTimer.trigger);
    clearTimeout(schedInProg);
    clearTimeout(scheduleTimer);
    clearInterval(progressIndicatorTimer);
}


function buildStationSelect() {
    var menu = document.getElementById("station");
    menu.innerHTML="";

    var national = document.createElement("optgroup");
    national.label = "National";
    var regional = document.createElement("optgroup");
    regional.label = "Regional";
    var local = document.createElement("optgroup");
    local.label = "Local";
    
    var stationToSelect = currentStation.nickname+(currentStation.extra ? currentStation.extra : "");

    for (key in stationList) {
        var station = (stationList[key]);
        var newStation = document.createElement("option");
        newStation.value = key;
        newStation.innerHTML = station.name;
        if (key==stationToSelect) newStation.selected = true;
        if (station.coverage=="national") {
            national.appendChild(newStation);
        } else if (station.coverage=="regional") {
            regional.appendChild(newStation);
        } else {
            local.appendChild(newStation);
        }
    }
    menu.appendChild(national);
    menu.appendChild(regional);
    menu.appendChild(local);
}


// onShow() runs whenever Dashboard loads. We use it to refresh the
// schedule information in case it's out of date.
function onShow()
{
	// refresh the schedule & force a scroll to the programme
	refreshSchedule("forceScroll");
    // check play state
    if (widget.preferenceForKey("WaitingForRestart")) {
        if (!justShownRestartMessage) showStatus(null,'Widget upgraded! Please reload (&#x2318;R) the widget or click this message to restart Dashboard.',30,function(){widget.system("killall Dock; sleep 3; [[ -e /Applications/Dashboard.app ]] && open /Applications/Dashboard.app", killDashboardHandler);});
        justShownRestartMessage = false;
    }
    // check to see what the latest version is; at intervals of a day, this
    // function will check online, otherwise it uses a local cache of the last
    // known value.
    checkForUpdate();
}


// onHide() runs whenever Dashboard is hidden. It stops the timer
// calling refreshSchedule() and refreshing the scedule view.
function onHide()
{
	clearTimeout(scheduleTimer);
	scheduleTimer = null;
    // workaround for an audio loss that could occur when leaving Dashboard
    // before before Flash is placed
    if (flashTimer.trigger != null && flashTimer.seconds <= 4) {
        changeStream();
        alert("did it");
    }
}


// called from onLoad() to fill in the author & version information on the rear;
// it grabs them from Info.plist
function fillVersion()
{
    // start a new XMLHttpRequest
    var internalXmlRequest = new XMLHttpRequest();
    // setup the destination of the request (Info.plist)
    internalXmlRequest.open("GET", "Info.plist", false);
    // send nothing
    internalXmlRequest.send(null);
    // get all the "key" tags, set them to an array(?)
    var keys = internalXmlRequest.responseXML.getElementsByTagName("key");
    // set the variable to 0.0, so we have a value just in case this fails
    var temp_currentVersion = 0;
    // set up a loop, to go through every key tag
    for(i=0; i<keys.length; i++)
    {
        // if we hit a match for CFBundleVersion in the data of the "first
        // child" of any of the keys (XML navigation terminology is rather
        // confusing)
        if("CFBundleVersion" == keys[i].firstChild.data)
        {
            // if we hit CFBundleVersion, we know where the actual version
            // number will be inrelation to it (assuming the .plist is valid)
            // so we can navigate to it - again, confusing-looking, no?
            // Anyway, we set it to the global varible, so we can refer to it
            // in other functions
            temp_currentVersion = keys[i].nextSibling.nextSibling.firstChild.data;
            // break the loop, we got a match so we're done
            break;
        }
    }
    // uses the newly-set variable to fill in the version number, along with
    // the rest of the information, into the span with id="author"
	document.getElementById("author").innerHTML = "BBC Radio Widget v"+temp_currentVersion;
    currentVersion = splitVersion(temp_currentVersion);
    if (currentVersion.beta) { debug = true; }
    if (debug) alert("Constructed version number: "+versionDisplayString(currentVersion));
}


// constructs an object containing a complex version string,
// including an integer beta value if provided.
// takes a string, "X.xxbX" as an argument; X can be many digits
// long, but xx will be truncated to 2 digits.
// so eg. 4.9 (=4.90) > 4.11 > 4.108 (=4.10) > 4.02
function splitVersion(verString) {
    var ver = new Object();
    verString += "";
    if (verString.indexOf("b")!=-1) {
        ver.beta = parseInt(verString.split("b")[1]);
        if (isNaN(ver.beta)) ver.beta = null;
        verString = verString.substring(0,verString.indexOf("b"));
    } else {
        ver.beta = null;
    }
        
    if (verString.indexOf(".")!=-1) {
        var temp_minor = verString.split(".")[1];
        if (temp_minor.length==1) { temp_minor+="0"; }
        else if (temp_minor.length>2) { temp_minor=temp_minor.substr(0,2); }
        ver.minor = parseInt(temp_minor);
        if (isNaN(ver.minor)) ver.minor = 0;
        verString = verString.substring(0,verString.indexOf("."));
    } else {
        ver.minor = 0;
    }
    
    ver.major = parseInt(verString);
    if (isNaN(ver.major)) ver.major = 0;

    if (debug) alert("Version was created: "+versionDisplayString(ver));
    return ver;
}


function checkIfVersionHigher(baseline, contender) {

    if (baseline==undefined || contender==undefined) { return false; }

    // sanity check; we should never create versions
    // which fail, but just in case
    if (!baseline.major) baseline.major = 0;
    if (!baseline.minor) baseline.major = 0;
    if (!contender.major) contender.major = 0;
    if (!contender.minor) contender.major = 0;

    if (debug) alert("Comparing versions: "+versionDisplayString(baseline)+" and "+versionDisplayString(contender));
    
    if (contender.major>baseline.major) {
        return true;
    } else if (contender.major<baseline.major) {
        return false;
    }
    
    if (contender.minor>baseline.minor) {
        return true;
    } else if (contender.minor<baseline.minor) {
        return false;
    }

    // if we've reached this point, numbers match so
    // we're into checking if only one is a beta (it's older),
    // or comparing beta revisions
    if (baseline.beta && !contender.beta) {
        return true;
    } else if (!baseline.beta && contender.beta) {
        return false;
    }
    if (contender.beta>baseline.beta) {
        return true;
    } else if (contender.beta<baseline.beta) {
        return false;
    }
    
    return false;
}


// checks to see what the latest version is. Every two days it looks online at
// my website, where the latest version will be published; in between it uses
// a local cache of the last known value instead.
function checkForUpdate()
{
    // get the current date
    var now = (new Date).getTime();
    // if we haven't checked for an update, or if it's been a day. Used to only
    // allow this if an update hadn't been detected previously; however, this
    // doesn't allow for human error. I want to be able to pull new versions if
    // they're buggy and un-trigger the update check!
    if (!then || latestVersion.release.major==0 || (now-then)>86400000)
    {
        // cancel any update request in progress - there damn well shouldn't be
        // one, if everything's working!
        if (externalXmlRequest != null)
        {
            externalXmlRequest.abort();
            externalXmlRequest = null;
        }
        // set up the request
        externalXmlRequest = new XMLHttpRequest();
        // when we've got the data, this calls processVersionResponse()
        // to handle it, passing the data to it
        externalXmlRequest.onload = function(e) {processVersionResponse(e, externalXmlRequest);}
        // gives the url to query, amongst other things; if you want to see
        // how the other end works, visit that URL in your browser and
        // view the source
        externalXmlRequest.open("GET", "http://www.phantomgorilla.com/update/versions.php?product=BBCRadio&version="+versionDisplayString(currentVersion)+"&protocol=1&devmode="+(window.widget ? 'N' : 'Y'), true);
        // make sure we don't get a cached version of the page
        externalXmlRequest.setRequestHeader("Cache-Control", "no-cache");
        // don't send anything
        externalXmlRequest.send(null);
    }
    determineUpdateAvailability();
}


// searches the nodes in the page for the one that relates to this product, then
// gets the data from it - the latest version number :)
function processVersionResponse(e, request)
{
    // set the externalXmlRequest to null, so we know no request is in progress
    externalXmlRequest = null;
    // if there's an answer with text
    if (request.responseText && request.status && request.status==200)
    {
        // We got a response, so let's note this time as the most recent check
        then = (new Date).getTime();
        if (window.widget) widget.setPreferenceForKey(then,"lastupdatecheck");
        
        if (debug) alert(request.responseText);
        var response = JSON.parse(request.responseText);
        var release = response.products.BBCRadio.release;
        var beta = response.products.BBCRadio.beta;
        if (release.version) {
            latestVersion.release = splitVersion(release.version);
            if (release.url) {
                updateUrl = release.url;
            } else {
                updateUrl = null;
            }
        }
        if (currentVersion!=undefined && currentVersion.beta && beta.version) {
            latestVersion.betaRelease = splitVersion(beta.version);
            if (beta.url) {
                updateUrl = beta.url;
            } else {
                updateUrl = null;
            }
        }
        // stores the latest version information we just got as a preference;
        // even though latestVersion is a global variable, we save this now so
        // that on next load (when it will have cleared) we'll still know what
        // the latest version is without checking online (till a day is up)
        if (window.widget) {
            widget.setPreferenceForKey(versionDisplayString(latestVersion.release), "latestversion");
            widget.setPreferenceForKey(versionDisplayString(latestVersion.betaRelease), "LatestBeta");
            widget.setPreferenceForKey(updateUrl, "UpdateUrl");
        }
    }
    determineUpdateAvailability();
}


// does some stuff depending on whether an update is available
function determineUpdateAvailability() {
    // now we've got the latest version information, we have to check to see if
    // there's actually an update, or if this version is current.
    // if the latest version available is greater than the current version:
    if (checkIfVersionHigher(currentVersion, latestVersion.release)) {
        // hide the Phantom Gorilla graphic in the lower left
        document.getElementById("phantom").style.display="none";
        // show the "update" graphics instead
        document.getElementById("update").style.display="block";
        document.getElementById("front_update").style.display="block";
        if (debug) alert("*** Update found! ***");
    }
    else if (currentVersion!=undefined && currentVersion.beta && checkIfVersionHigher(currentVersion, latestVersion.betaRelease)) {
        // hide the Phantom Gorilla graphic in the lower left
        document.getElementById("phantom").style.display="none";
        // show the "update" graphics instead
        document.getElementById("update").style.display="block";
        document.getElementById("front_update").style.display="block";
        if (debug) alert("*** Update found! ***");
    }
    // otherwise, do the reverse (in case the update availability has reverted
    // while the widget is open; unlikely, but possible
    else {
        document.getElementById("update").style.display="none";
        document.getElementById("front_update").style.display="none";
        document.getElementById("phantom").style.display="block";
        if (debug) alert("No update found");
    }
}


function openUpdatePage() {
    var the_url;
    if (updateUrl && updateUrl.length > 7) {
        the_url = updateUrl;
    } else {
        the_url = "http://www.phantomgorilla.com/widgets/";
    }
    if (window.widget) {
        widget.openURL(the_url);
    } else {
		window.location.href = the_url;
    }
}


// Originally changestation(), but it evolved into a behemoth prefs function.
// setPrefs() is called whenever a station is chosen from the popup menu.
// It saves the location of the image and the stream url of the corresponding
// station and then calls functions which change the image and the audio stream
// on the front of the widget.
function setPrefs()
{
    var elem = document.getElementById("station");
    // look up the station in the stationList object.
    var selectedOption=elem.options[elem.selectedIndex].value;
    currentStation = stationList[selectedOption];

    if (window.widget) {
        widget.setPreferenceForKey(selectedOption,"CurrentStation");
        widget.setPreferenceForKey(selectedOption,makeKey("CurrentStation"));
    }
        
    changeImage();
} 


// changeImage() is called on loading and after preference changes.
// It checks which images should be shown and changes the html to show it.
// Also adds any additional info such as local radio area over the image,
// if appropriate.
function changeImage()
{
    // naughty check for hidden key "UseOfficialLogos"
    var useOfficial = (window.widget ? widget.preferenceForKey("UseOfficialLogos") : false);
    if (useOfficial) {
        useOfficial = true;
        document.getElementById("main").className = "official";
    } else {
        useOfficial = false;
        document.getElementById("main").className = "";
    }
    
    var logo = currentStation.logo;
    // check for valid preference
    if (logo && (logo.length>0)) {
        // set the source of the image correctly
        if (useOfficial) {
        // more naughtiness; switch to official logos if hidden key told us to
            document.getElementById("logo").src = "images/official/"+logo;
        } else {
        // use normal logos otherwise
            document.getElementById("logo").src = "images/normal/"+logo;
        }
    }
    // if this is local radio
    if (currentStation.coverage == "local") {   
        var localContainer = document.getElementById("local_name_container");
        // display the name of the local station on the front
        if (currentStation.shortname) {
            localContainer.innerText = currentStation.shortname;
        } else {
            localContainer.innerText = currentStation.name;
        }
        localContainer.style.display="block";  
        // and hide the LW/FM indicator as it's redundant here
        document.getElementById("extrainfo_container").style.display="none";
    }
    // otherwise, if there are multiple station bands/variants
    else if (currentStation.extra && currentStation.extra.length > 0) {
        var extraInfo = document.getElementById("extrainfo_container");
        // show FM/LW info
        extraInfo.innerText = currentStation.extra;
        extraInfo.style.display="block";
        // hide the local info, it's redundant
        document.getElementById("local_name_container").style.display="none";
    }
    else {
        // turn off both if neither needed
        document.getElementById("local_name_container").style.display="none";
        document.getElementById("extrainfo_container").style.display="none";
    }
}


// changeStream() - formerly changeStreamUrl() - is called when the play button
// is pressed, or after preferences are changed. It checks which stream should
// be playing, and then composes some html which embeds the real player plugin
// in the widget to stream the audio. Then the html is pasted into the widget.
function changeStream()
{
    if (debug) alert("Attempting to use: "+useType);
    // if we want Flash and it's available
    // or there's no RealPlayer link
    if (useType == "flash") {
        placeFlash(currentStation.flash, true);
    } else if (useType == "html5") {
        placeHtml5(currentStation.html5);
    } else if (useType == "wma") {
        placeWma(currentStation.wma);
    }
}


// places the wma player
function placeHtml5(url) {
    var html = '<audio id="html5_player" src="'+url+'" autoplay>';
    // change the inner html of the radio span to the html we just composed
    document.getElementById("radio").innerHTML = html;
    // set the player to the volume the slider is showing
    setTimeout("changeVolume();", 100);
    document.getElementById("html5_player").addEventListener('play', function(e){stopProgressIndicator();},false);

    document.getElementById("html5_player").addEventListener('canplay', function(e){Html5Error('canplay')},false);
    document.getElementById("html5_player").addEventListener('play', function(e){Html5Error('play')},false);
    document.getElementById("html5_player").addEventListener('stalled', function(e){Html5Error('stalled')},false);
    document.getElementById("html5_player").addEventListener('pause', function(e){Html5Error('pause')},false);
    document.getElementById("html5_player").addEventListener('error', function(e){Html5Error('error',e)},false);
    document.getElementById("html5_player").addEventListener('ended', function(e){Html5Error('ended')},false);
    document.getElementById("html5_player").addEventListener('emptied', function(e){Html5Error('emptied')},false);
    document.getElementById("html5_player").addEventListener('abort', function(e){Html5Error('abort')},false);
}

function Html5Error(eType,e) {
    if (debug) alert("Html5 event: "+eType);
    if (eType=='ended') {
        pauseClicked();
        playClicked();
    }
    if (eType=='error') {
       switch (e.target.error.code) {
         case e.target.error.MEDIA_ERR_ABORTED:
           alert('You aborted the video playback.');
           break;
         case e.target.error.MEDIA_ERR_NETWORK:
           alert('A network error caused the audio download to fail.');
           break;
         case e.target.error.MEDIA_ERR_DECODE:
           alert('The audio playback was aborted due to a corruption problem or because the video used features your browser did not support.');
           break;
         case e.target.error.MEDIA_ERR_SRC_NOT_SUPPORTED:
           alert('The audio could not be loaded, either because the server or network failed or because the format is not supported.');
           noHTML5 = true;
           pauseClicked();
           playClicked();
           break;
         default:
           alert('An unknown error occurred.');
           break;
       }
    }
}

// places the wma player
function placeWma(url) {
    var html = '<OBJECT CLASSID="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" WIDTH="1" HEIGHT="1" CODEBASE="http://www.apple.com/qtactivex/qtplugin.cab" id="wma_player"><PARAM name="SRC" VALUE="'+url+'"><PARAM name="AUTOPLAY" VALUE="true"><PARAM name="CONTROLLER" VALUE="false"><PARAM name="ENABLEJAVASCRIPT" VALUE="true"><PARAM name="SHOWLOGO" VALUE="false"><PARAM name="POSTDOMEVENTS" VALUE="true"></EMBED></OBJECT>';
    // change the inner html of the radio span to the html we just composed
    document.getElementById("radio").innerHTML = html;
    // set the player to the volume the slider is showing
    setTimeout("changeVolume();", 100);
    document.getElementById("wma_player").addEventListener("qt_error", qtError, false);
    document.getElementById("wma_player").addEventListener("qt_pause", qtPause, false);
    document.getElementById("wma_player").addEventListener("qt_stalled", qtStalled, false);
    document.getElementById("wma_player").addEventListener("qt_play", qtPlay, false);
    document.getElementById("wma_player").addEventListener("qt_progress", qtProgress2, false);
    qtHasLoaded = false;
    waitThenCancel = setTimeout((debug ? "alert(\"No response received\"); " : "")+"pauseClicked();", 30000);
}

function qtError(event) {
    pauseClicked();
    if (debug) alert("Quicktime error");
}

function qtPause(event) {
    if (debug) alert("Quicktime paused");
}

function qtStalled(event) {
    document.getElementById("wma_player").addEventListener("qt_progress", qtProgress, false);
    startProgressIndicator('large');
    qtProgressCount = 0;
    waitThenCancel = setTimeout("restartQt();", 12000);
    if (debug) alert("Quicktime stalled");
}

function restartQt() {
    if (debug) alert("Restarting Quicktime playback");
    document.getElementById("wma_player").removeEventListener("qt_progress", qtProgress, false);
    pauseClicked();
    playClicked();
}

var qtHasLoaded = false;
var qtProgressCount = 0;
var waitThenCancel = null;

function qtPlay(event) {
    // do nothing if we're still loading
    if (document.getElementById("wma_player").GetPluginStatus()!="Complete") {
        return;
        if (debug) alert("Quicktime played (not loaded before)");
    }
    
    // if we've gone from unloaded to loaded, stop the spinning indicator as we're playing now
    if (!qtHasLoaded) {
        stopProgressIndicator();
        qtHasLoaded = true;
        if (debug) alert("Quicktime played (after loading)");
        return;
    }
    // otherwise we were loaded before, something may be wrong.
    // wait 8s to see if we're still loading data, cancel if we aren't
    // (unless we're already waiting)
    alert(waitThenCancel);
    if (waitThenCancel==null) {
        waitThenCancel = setTimeout("restartQt();", 8000);
    }
    if (debug) alert("Quicktime played (during playback)");
}

// counts pieces of data loaded, up to 2
// used to check if a stall is permanent
function qtProgress(event) {
    qtProgressCount++;
    if (debug) alert("Quicktime progress: data "+qtProgressCount);
    if (qtProgressCount >= 2) {
        // we now know 3 pieces of data loaded since the stall was detected
        // so we assume it's not permanent and reset our display
        document.getElementById("wma_player").removeEventListener("qt_progress", qtProgress, false);
        if (qtHasLoaded) stopProgressIndicator();
        // then rset the count, and stop waiting to cancel playback (if we are)
        qtProgressCount = 0;
        if (waitThenCancel!=null) {
            clearTimeout(waitThenCancel);
            waitThenCancel = null;
        }
        if (debug) alert("Playback stop cancelled due to successful resume");
    }
}

function qtProgress2(event) {
    if (waitThenCancel) {
        clearTimeout(waitThenCancel);
        waitThenCancel = null;
    }
    document.getElementById("wma_player").removeEventListener("qt_progress", qtProgress2, false);
}

/*****************************************/
// HANDLING PLAYER INTERFACE MOUSE EVENTS
/*****************************************/


// called when the mouse clicks on certain objects.
// we get passed the reference of the object on which the mouse went down,
// and an english reference for it
function theMouseDownHandler(event, div, clickedElement)
{
    // loads each image we could use into a cariable, so that they're cached
    // and won't flicker when we swap them in
	var img = new Image;
	img.src = "images/play2_clicked.png";
	img.src = "images/play2.png";
	img.src = "images/pause2_clicked.png";
	img.src = "images/pause2.png";
	img.src = "images/knob_clicked.png";
	img.src = "images/knob.png";

    // passes the clickedElement to the global variable
    // wasClickedElement so the other functions can use it
    wasClickedElement=clickedElement;

    // if we clicked on the play button
    if (wasClickedElement=="play")
    {
        // change to the clicked play button image
        document.getElementById("play").src="images/play2_clicked.png";
    }
    // if we clicked on the pause button
    if (wasClickedElement=="pause")
    {
        // change to the clicked play button image
        document.getElementById("pause").src="images/pause2_clicked.png";
    }
    // if it's the volume slider/knob that was clicked
    else if (wasClickedElement=="knob" || wasClickedElement=="slider")
    {
        if (useType=="flash") return;
        // change to the clicked knob image
        document.getElementById("knob").src="images/knob_clicked.png";
        // Save starting positions of cursor and element.
        cursorStartX = event.clientX;
        knobStartLeft = parseInt(document.getElementById("knob").style.left, 10)
	}
    // if it's the volume slider, move the knob to point of click
    if (wasClickedElement=="slider")
    {
        // allowing for the size of the knob (we want it centred over our click,
        // but we position it using the left hand side) move it all the way to
        // the left if user clicked on the left boundary (global, set at start)
        if ((cursorStartX-6) < leftBound)
        {
            var skipKnobToHere = leftBound;
        }
        // same as above, but for right boundary (also set at start)
        else if ((cursorStartX-6) > rightBound)
        {
            var skipKnobToHere = rightBound;
        }
        // otherwise, skip the knob to where we clicked; again, allowing for the
        // size & the fact that we want to centre the knob over our click
        else
        {
            var skipKnobToHere = (cursorStartX-6);
        }
        // do the actual moving via setting the CSS property
        document.getElementById("knob").style.left = skipKnobToHere + "px";
        // reset the knob's starting position to the new position, in case
        // we do some dragging after the click
        // Are you impressed that this is allowed? ;) I'm quite proud of this!
        knobStartLeft = parseInt(document.getElementById("knob").style.left, 10)
        // calls the function to change the volume, based on the new position
        changeVolume();
	}
	// add listeners, to trigger handlers if the mouse does anything
	document.addEventListener("mousemove", theMouseMoveHandler, true);
	document.addEventListener("mouseup", theMouseUpHandler, true);
	div.addEventListener("mouseover", theMouseOverHandler, true);
	div.addEventListener("mouseout", theMouseOutHandler, true);
	// set it up so we know globally that the mouse is down, and where
	div.arrowInside = true;
	theCurrentMouseDownButton = div;
    // I dunno, but Apple do this in their button handlers (from which all these
    // are derived)
	event.stopPropagation();
	event.preventDefault();
}


// when the mouse moves after mousedown, this is called
function theMouseMoveHandler (event)
{
    // if we clicked on the knob or the slider; ie, we're in a drag
    if (wasClickedElement=="knob" || wasClickedElement=="slider")
    {
        if (UseFlash) return;
        // calculate the knob position based on its start point and how much
        // the cursor has moved
        var newPosition = (knobStartLeft + event.clientX - cursorStartX);
        // if that's within the slider boundaries, move there
        if (newPosition > leftBound && newPosition < rightBound)
        {
            document.getElementById("knob").style.left = newPosition + "px";
        }
        // if it's beyond the left boundary (set globally, at start)
        else if (newPosition < leftBound)
        {
            // just skip to the left boundary instead
            document.getElementById("knob").style.left = leftBound + "px";
        }
        // if it's beyond the right boundary (again, set globally, at start)
        else if (newPosition > rightBound)
        {
            // skip to the right boundary
            document.getElementById("knob").style.left = rightBound + "px";
        }
        // calls changeVolume() to update the volume based on these moves
        changeVolume();
	}
    // like I said, Apple use these
	event.stopPropagation();
	event.preventDefault();
}


// this is called if the mouse goes back over the element - obviously, we must
// already have had a mouseout event for this to be being called!
function theMouseOverHandler (event)
{
    // if the mouse button's still down, and the original click was on the button
	// (not the knob or slider, we don't let the knob change
    // back from blue till the mouse button is released, so there's no point
    // in calling them here again)
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // change to the clicked images
        document.getElementById("play").src="images/play2_clicked.png";
	}
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // change to the clicked images
        document.getElementById("pause").src="images/pause2_clicked.png";
	}
	// set variable to show the mouse is currently over the element clicked on
	theCurrentMouseDownButton.arrowInside = true;
    // getting bored of saying "I don't know"
	event.stopPropagation();
	event.preventDefault();
}


// called if the mouse moves away from the element clicked on
function theMouseOutHandler (event)
{
	// if original click was on play button
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // switch to the unclicked image, as the mouse isn't over it
        document.getElementById("play").src="images/play2.png";
	}
	// and the pause button
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // switch to the unclicked image, as the mouse isn't over it
        document.getElementById("pause").src="images/pause2.png";
	}
    // set variable to indicate the mouse is outside the element clicked on
	theCurrentMouseDownButton.arrowInside = false;
	// je ne sais pas
	event.stopPropagation();
	event.preventDefault();
}


// when the button is released, this is called - the business part of all this
function theMouseUpHandler (event)
{
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // put the unclicked images back, the mouse is up!
        document.getElementById("play").src="images/play2.png";
	}
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // put the unclicked images back, the mouse is up!
        document.getElementById("pause").src="images/pause2.png";
	}
	else if (theCurrentMouseDownButton && (wasClickedElement=="knob" || wasClickedElement=="slider"))
	{
        if (UseFlash) return;
        // put the unclicked images back, the mouse is up!
        document.getElementById("knob").src="images/knob.png";
	}
	// callback to the client, clean up
	document.removeEventListener("mousemove", theMouseMoveHandler, true);
	document.removeEventListener("mouseup", theMouseUpHandler, true);
	if (theCurrentMouseDownButton)
	{
		theCurrentMouseDownButton.removeEventListener("mouseover", theMouseOverHandler, true);
		theCurrentMouseDownButton.removeEventListener("mouseout", theMouseOutHandler, true);
	}
    // Quintus est puer Romanus
	event.stopPropagation();
	event.preventDefault();
    // IF the button was depressed over an element and it's now been
	// released *over the same element*
	if (theCurrentMouseDownButton &&
	    theCurrentMouseDownButton.arrowInside)
	{
    // if it was the play button clicked, call a function
	if (wasClickedElement=="play")
        {
            // this places the plugin (unless audio is already playing)
            playClicked(event);
        }
    // if it was the pause button instead, call another function
	else if (wasClickedElement=="pause")
        {
            // this removes the plugin
            pauseClicked();
        }
    }
	// reset the variable, as the button's been released
	theCurrentMouseDownButton = null;
}


// openWhatsOnPage() tells the widget to open the appropriate schedule in
// a browser window
function openWhatsOnPage()
{
    // read the preference for the nickname of the station - a component
    // of the url we use that's specific to each station.
    var nickname = currentStation.nickname;
    // check for a pref
    if (nickname && (nickname.length > 0))
    {
        // compose
        var schedulePage = "http://www.bbc.co.uk/"+nickname+"/programmes/schedules";
        // tweak for weird stations
        if (nickname=="radio1") schedulePage += "/england";
        if (nickname=="radio4") schedulePage += "/" + currentStation.extra.toLowerCase();
        if (nickname=="radioscotland") schedulePage += "/fm";
        if (nickname=="radiowales") schedulePage += "/fm";
        // open the link
        if (window.widget) { widget.openURL(schedulePage); } else { window.open(schedulePage); }
    }
}


function canHTML5() {
	var elem = document.createElement('audio');
	if(typeof elem.canPlayType == 'function'){
		var playable = elem.canPlayType('audio/aac');
		var playable2 = elem.canPlayType('audio/mp4');
		var playable3 = elem.canPlayType('audio/x-scpls');
		if (debug) alert("html5 claims: aac "+playable+" / mp4 "+playable2+" / pls "+playable3);
		if(((playable.toLowerCase() == 'maybe')||(playable.toLowerCase() == 'probably'))
		&& ((playable3.toLowerCase() == 'maybe')||(playable3.toLowerCase() == 'probably'))){
			return true;
		}
	}
	return false;
};


// called when the play button is clicked (from theMouseUphandler), places the
// plugin
function playClicked(e)
{
    // we can disable tests with a plist key, or case-by-case with alt.
    // combine both possibilities for ease of use.
    if ((e && e.altKey) || disableTests) {
        var noTests = true;
    } else {
        var noTests = false;
    }

    // hide any status messages showing
    if (document.getElementById("status").innerHTML!="") {
        clearTimeout(statusTimeout);
        statusTimeout=null;
        hideStatus();
    }
    // clear flash timer details
    if (flashTimer.trigger) {
        clearTimeout(flashTimer.trigger);
        flashTimer.trigger = null;
        flashTimer.seconds = 0;
    }
    // start the progress indicator running (so long as it's 
    // currently stopped, otherwise we'd make the animation
    // restart and it'd look horrible)
    if (progressIndicatorTimer==null)
    {
        startProgressIndicator('large');
    }
    // hide the play button, show the pause button
    document.getElementById("play").style.display="none";
    document.getElementById("pause").style.display="block";
    // skip if tests disabled
    if (noTests) {
        changeStream();
    } else {
        // pinched this idea from the BBC ListenLive widget; hope Duncan Ponting
        // doesn't mind. It's a great trick!
        var flashHere = false;
        var wmaHere = false;
        var html5Here = false;
        // loop through all the plugins
        for (var i=0;i < navigator.plugins.length; i++)
        {
            // check for an appropriately named plugin
            if (navigator.plugins[i].name.toLowerCase().indexOf("flash")!=(-1)) {
                flashHere = true;
            }
            if (navigator.plugins[i].name.toLowerCase().indexOf("flip4mac windows media")!=(-1)) {
                wmaHere = true;
            }
        }
        html5Here = canHTML5();
        if (debug) alert("Found plugins: Flash "+flashHere+", Flip4Mac "+wmaHere+", HTML5 "+html5Here);
        // choose what to use
        if (html5Here && currentStation.html5 && noHTML5!==true) {
            useType = "html5";        
        } else if (wmaHere && currentStation.wma) {
            //useType = "wma"; // disabled as doesn't work as of 2014. silverlight maybe?
        } else if (flashHere && currentStation.flash) {
            useType = "flash";
        } else {
            useType = "none";
        }
        // if we have no type to use
        if (useType=="none")
        {
            // show a status message, with link
            showStatus(e,'Please upgrade to Mac OS X 10.6 or higher, or click<br>to install Flash (widget<br>must be restarted).',20,function(){javascript:widget.openURL("http://get.adobe.com/flashplayer/")});
            // pretend pause is clicked, so we stop the indicator and reset the
            // button images
            pauseClicked();
            return;
        }
        if (useType=="flash" && currentStation.html5
            && (new Date).getTime()-lastShowFlashWarning >= 60*60*1000)
        {
            showStatus(e,'Using Flash. AAC (better) requires Mac OS X 10.6 with Safari 4 or higher.',10,null);
            lastShowFlashWarning = (new Date).getTime();
        }
        // udate volume display
        checkForFlash();
        // if the plugin's installed
        // use xmlhttprequest to test for internet connection - if we can complete
        // a connection to the BBC radio player page we proceed and place the player
        // (if we place it without a connection we could get a crash later!)
        // set up the request
        xmlhttp = new XMLHttpRequest();
        // location et al
        xmlhttp.open("HEAD", "http://www.bbc.co.uk/radio/",true);
        //xmlhttp.open("HEAD", "http://google.com",true);
        // every time the state of the request changes
        // run the following code
        xmlhttp.onreadystatechange = processTestResponse;
        // send nothing
        xmlhttp.send(null);
        processTestResponse;
    }
}


// Processes the state changes of the connection test. Ugly workaround for
// Safari 4 beta's poor handling of XMLHttpRequests when there's no internet
// connection – onreadystatechange doesn't progress properly so we explicitly
// check.
function processTestResponse()
{
    if (xmlhttp && xmlhttp.readyState && xmlhttp.readyState==1) {
        // timeout test is explained more thoroughly in schedule.js
        if (testInProg && testInProg!=null) {
            clearTimeout(testInProg);
            testInProg = null;
            showStatus(null,"You don’t appear to be connected to the internet. Please check your connection and try again.", 10);
            clearTimeout(statusTimeout);
            statusTimeout = setTimeout('showStatus(null,"If you’re certain you’re connected, option-click<br>(⌥-click) the Play button<br>to override this test.", 7)',10000);
            pauseClicked();
        } else {
            testInProg = setTimeout("processTestResponse();", 10000);
        }
    }
    // if the connection request is complete
    else if (xmlhttp && xmlhttp.readyState && xmlhttp.readyState==4)
    {
        //alert("st: "+xmlhttp.status+" "+xmlhttp.statusText);
        clearTimeout(testInProg);
        testInProg = null;
        // and if the request was successful
        if (xmlhttp.status==200)
        {
            // place an instance of the plugin, change the volume,
            // change the play button to a pause button
            changeStream();
        }
        // if unsuccessful
        else
        {
            // calls a function to show a "no connection" warning
            // really ugly hack to stop this appearing when the user cancels
            if (document.getElementById("pause").style.display=="block") {
                showStatus(null,"You don’t appear to be connected to the internet. Please check your connection and try again.", 10);
            }
            // pretend pause is clicked, so we stop the indicator and
            // reset the button images
            pauseClicked();
        }
    }
}


// Called when pause is clicked, to remove the plugin and stop audio
function pauseClicked()
{
    if (debug) alert("Stopping playback");

	// stop the progress indicator, in case it was running
    stopProgressIndicator();
    // resets the inner html of the span holding the plugin
	// and resets the flash timeout (otherwise if we hide
    // Dashboard too soon the flash placement audio loss
    // workaround might cause audio to start again)
    clearInterval(flashTimer.trigger);
    flashTimer.seconds = 0;
    flashTimer.trigger = null;
    // clear some wma stuff
    if (waitThenCancel!=null) {
        clearTimeout(waitThenCancel);
        waitThenCancel = null;
    }
    qtHasLoaded = false;
    qtProgressCount = 0;

    document.getElementById("radio").innerHTML="";
    // hide the pause button, shaow the play button
    document.getElementById("pause").style.display = "none";
    document.getElementById("play").style.display = "block";
    // abandon connection test timeout
    if (testInProg && testInProg!=null) {
        clearTimeout(testInProg);
        testInProg = null;
    }
	// stop the xmlhttprequest - on slow connections it might not have
	// completed yet and it can lead to audio starting after pause has
	// been clicked.
	if (xmlhttp!=null)
	{
		xmlhttp.abort();
		xmlhttp = null;
    }
    // double check for Flash use
    checkForFlash();
}


// show a status message
function showStatus(e, message, duration, clickhandler)
{
    if (!clickhandler) var clickhandler = function () { return;};
    var statusContainer = document.getElementById("status");
	// stop any status fade event that's due to be called
	clearTimeout(statusTimeout);
	statusTimeout = null;
    // pass the event to a global variable (we can't pass it on to hideStatus()
    // because we call it via setTimeout(), which doesn't support that, so we
    // have to make it global)
    globalStatusEvent=e;
    // set the inner html to the message we got passed
    statusContainer.innerHTML=message;
    // set the onclick
    statusContainer.onclick= clickhandler;
    // show the status div (although it's transparent at the moment)
    statusContainer.style.display="block";
    // setting up an AppleAnimator, to fade in the status message (speedily)
    // starting from 0 opacity
    var from=0;
    // if it's running
    if(null != statusAnimator)
    {
        // stop
        statusAnimator.stop();
        // start from current value
        from = statusAnimator.animations[0].now;
    }
    // if the shift key is down, the multiplier is 10, if not, 1
    if (!e || e==null) { var multiplier = 1 } else {
        var multiplier = e.shiftKey ? 10 : 1;
    }
    // upshot of this is, increasing values get passed to statusFadeHandler
    statusAnimator = new AppleAnimator(500*multiplier, 13, from, 1, statusFadeHandler);
    // starts it all off!
    statusAnimator.start();
    // call hideStatus after the amount of time we specified, to give plenty of
    // time to read it (we multiply by 1000 as 'duration' is specified in
    // seconds, but setTimeout uses thousanths of a second)
    duration = (duration*1000);
    statusTimeout = setTimeout('hideStatus()',duration);
}


// hides the status div, called after 3 seconds from the showStatus() function
function hideStatus()
{
    // setting up an AppleAnimator, to fade in the status message speedily
    // starting from 1 opacity
    var from = (document.getElementById("status").style.opacity ? document.getElementById("status").style.opacity : 1);
    // if it's running
    if(null != statusAnimator)
    {
        // stop
        statusAnimator.stop();
        // start from current value
        from = statusAnimator.animations[0].now;
    }
    // if the shift key is down, the multiplier is 10, if not, 1
    if (!globalStatusEvent || globalStatusEvent==null) { var multiplier = 1 } else {
        var multiplier = globalStatusEvent.shiftKey ? 10 : 1;
    }
    // upshot of this is, decreasing values get passed to statusFadeHandler
    statusAnimator = new AppleAnimator(500*multiplier, 13, from, 0, statusFadeHandler);
    // starts it all off!
    statusAnimator.start();
}


// handles the fading in and out of the status div
function statusFadeHandler(animation, current, from, to)
{
    // set the current opacity to what the Animator passes it
    document.getElementById("status").style.opacity=current;
    if (current==0)
    {
        // hide the status div (although it's transparent now)
        document.getElementById("status").style.display="none";
        // reset message to nothing
        document.getElementById("status").innerHTML="";
    }
}


// changes the volume when called (when placing a new plugin or when changing
// the slider position), based on where the slider knob is
function changeVolume()
{
    clearTimeout(volumeCounter);
    // get the knob position
    var sliderPosition = parseInt(document.getElementById("knob").style.left, 10);
    // the start of the slider is several pixels in from the left, so deduct those;
    // also the slider lingth is 33, so divide to get value out of 100
    var newVolume = (sliderPosition-leftBound)*100/32
    //if (debug) alert("Setting volume "+newVolume);
    // if there's a plugin instance currently placed
    if (document.getElementById("wma_player")) {
        document.getElementById("wma_player").SetVolume(newVolume*2.56);
    }
    if (document.getElementById("html5_player")) {
        document.getElementById("html5_player").volume = (newVolume/100);
    }
    volumeCounter = setTimeout("saveVolume("+newVolume+");",500);
}


function saveVolume(vol) {
    // if we're in Dashboard
    if (window.widget) {
        // save this as a preference (we'll use it to determine where the slider
        // should be on next launch)
        widget.setPreferenceForKey(vol,"Volume");
        widget.setPreferenceForKey(vol,makeKey("Volume"));
    }
}


// called at launch, to position the slider according to the volume level we've
// got saved in preferences
function restoreVolume()
{
    // if we're in Dashboard
    if (window.widget)
    {
        // retrieve the preference for this instance
        var newVolume = widget.preferenceForKey(makeKey("Volume"));
        // if it doesn't exist or make sense, load the global setting and overwrite it
        if (newVolume==undefined || newVolume.length < 0  || newVolume > 100) {
            newVolume = widget.preferenceForKey("Volume");
            if (newVolume==undefined || newVolume.length < 0 || newVolume > 100) {
                newVolume = 100;
                widget.setPreferenceForKey(newVolume,"Volume");
            }
            widget.setPreferenceForKey(newVolume,makeKey("Volume"));
        }
        if (newVolume>=0) {
            // get a value out of 32 (the slider length); then
            // add leftBound to this (the slider offset from the left margin) so we get
            // an absolute position for the knob
            var sliderPosition = (newVolume*32/100)+leftBound;
            // set the knob to this position
            document.getElementById("knob").style.left = sliderPosition;
        }
    }
}



/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/


// showPrefs() is called when the preferences flipper is clicked upon.  It
// freezes the front of the widget, hides the front div, unhides the back div,
// and then flips the widget over.
function showPrefs(event)
{
	// stop the schedule from periodically refreshing - we can't see it!
	clearTimeout(scheduleTimer);
	// check if the drawer is open, and close it if it is
	if (drawerStatus=="open")
	{
        goingToPrefsWithDrawerOpen = true;
        showHideDrawer(event);
	} else {
        // get the front and back divs
        var front = document.getElementById("front");
        var back = document.getElementById("back");
        // if we're in Dashboard
        if (window.widget) {
            // double check for Flash use
            // necessary now we have multiple instances
            if (document.getElementById("radio").innerHTML=="") checkForFlash();
            // freezes the widget so that you can change it without the user noticing
            widget.prepareForTransition("ToBack");
        }
        // remember which station is selected now
        // so we only interrupt audio later if we've changed station
        preferenceBefore = currentStation.name;
        
        // hide the front, show the back
        front.style.display="none";
        back.style.display="block";
        // and flip the widget over
        if (window.widget)
        {  setTimeout ('widget.performTransition();', 0);  }
	}
}


// hidePrefs() is called by the done button on the back side of the widget. It
// performs the opposite transition as showPrefs() does.
function hidePrefs()
{
    // get the front and back
	var front = document.getElementById("front");
	var back = document.getElementById("back");
    // if a new station has been selected, we change the audio stream playing;
    // otherwise we leave it alone so that playback isn't interrupted
    if (currentStation.name != preferenceBefore)
    {
        checkForFlash();
        // blank out the radio playback as soon as possible
        document.getElementById("radio").innerHTML = "";
        // pretend we clicked the play button, to start the audio
        playClicked(event);
		// get the new schedule
		getSchedule();
    }
	else
	{
	// refresh the current schedule
		refreshSchedule();
	}
	// freezes the widget and prepares it for the flip back to the front
    if (window.widget) widget.prepareForTransition("ToFront");
	// changes the image on the schedule drawer button
	updateDrawerButton();
	// hide the back
	back.style.display="none";
	// show the front
	front.style.display="block";
	// and flip the widget back to the front
	if (window.widget)
	{	setTimeout ('widget.performTransition();', 0);   }
	// if the drawer was open
	if (goingToPrefsWithDrawerOpen)
	{
		goingToPrefsWithDrawerOpen = false;
		setTimeout ('showHideDrawer();', 600);
	}
}


function checkForFlash() {
    if (useType=="flash") {
        document.body.className="flash";
        document.getElementById("volume").title="Sorry, changing volume is not available when using Flash.";
    } else {
        document.body.className="";
        document.getElementById("volume").title="";
    }
}


function placeFlash(url, autoplay) {
    if (autoplay!=true) autoplay = false;
    var html = '<div id="semp-wrapper" class="semp-wrapper"><embed type="application/futuresplash" src="http://www.bbc.co.uk/emp/10player.swf?revision=18182_18346" width="0" height="0" id="bbc_emp_embed_semp-wrapper" name="bbc_emp_embed_semp-wrapper" bgcolor="#000000" quality="high" wmode="default" allowfullscreen="false" allowscriptaccess="always" flashvars="config=http://www.bbc.co.uk/emp/iplayer/config.xml&amp;config_settings_showPopoutButton=false&amp;config_settings_autoPlay='+autoplay+'&amp;playlist='+url+'&amp;config_settings_showFullScreenButton=false"></div>';
    document.getElementById("radio").innerHTML = html;
	// reset timer etc
    flashTimer.seconds = 0;
    clearInterval(flashTimer.trigger);
    flashTimer.trigger = null;
    // run timer
    flashTimer.trigger = setInterval("flashCounter();", 1000);
}

function flashCounter() {
    flashTimer.seconds += 1;
    // after 10 seconds we assume audio has started
    if (flashTimer.seconds==10) {
        stopProgressIndicator();
        clearTimeout(flashTimer.trigger);
        flashTimer.trigger = null;
        flashTimer.seconds = 0;
    }
}

function checkForTestDisable() {
    // check for hidden key "DisableTests"
    var temp_DisableTests = (window.widget ? widget.preferenceForKey("DisableTests") : false);

    if (temp_DisableTests==true || temp_DisableTests==1 || temp_DisableTests=="true" || temp_DisableTests=="yes") {
        disableTests = true;
    }
}


// formats a version object for display
function versionDisplayString(version) {
    if (version!=undefined) {
    return version.major+"."+(version.minor<10 ? "0"+version.minor : version.minor)+(version.beta ? "b"+version.beta : "");
    } else {
    return "?.??";
    }
}


// doUpgrade is called whenever the last used version is
// older than the current version, in order that we can ditch
// old preferences and do other cleanup activities.
function doUpgrade(lastUsedVersion) {
    alert("Upgrading BBC Radio Widget from "+(lastUsedVersion ? "v"+versionDisplayString(lastUsedVersion) : "[unknown version]")+" to v"+versionDisplayString(currentVersion)+".");
    var stationPopup = document.getElementById("station");
    var optionToSelect = widget.preferenceForKey("selection");

    // if the last used version is under 4.1b1
    if (checkIfVersionHigher(lastUsedVersion,splitVersion("4.1b1"))) {
        if(debug) alert("Hi! The v4.1b1 upgrade code is running.");
        widget.setPreferenceForKey(currentStation.nickname,"CurrentStation");
        widget.setPreferenceForKey(currentStation.nickname,makeKey("CurrentStation"));
        widget.setPreferenceForKey(null,"logo");
        widget.setPreferenceForKey(null,"nickname");
        widget.setPreferenceForKey(null,"selection");
        widget.setPreferenceForKey(null,"url");
        widget.setPreferenceForKey(null,"area");
        widget.setPreferenceForKey(null,"FlashUrl");
        widget.setPreferenceForKey(null,"extraInfo");
        widget.setPreferenceForKey(null,"last_used_version");
        widget.setPreferenceForKey(null,"channel_id");
    }

    // if the last used version is under 4.5b3
    if (checkIfVersionHigher(lastUsedVersion,splitVersion("4.5b1"))) {
        if(debug) alert("Hi! The v4.5 upgrade code is running.");
        var oldVolume = widget.preferenceForKey(makeKey("Volume"));
        if (!oldVolume || oldVolume.length <= 0) {
            oldVolume = widget.preferenceForKey("Volume");
        }
        if (oldVolume) {
            var newVolume = oldVolume/32;
            if (newVolume>0.9) newVolume = 1;
            widget.setPreferenceForKey(newVolume,makeKey("Volume"));
            widget.setPreferenceForKey(newVolume,"Volume");
        }
    }
    // set this version as last used version
    widget.setPreferenceForKey(versionDisplayString(currentVersion),"LastUsedVersion");

    // helps us know if the widget has been cleanly upgraded
    //widget.setPreferenceForKey(true,"WaitingForRestart");
    //showStatus(null,'Widget upgraded! Please choose "Keep", and reload (&#x2318;R) or click this message to restart Dashboard.',30,function(){widget.system("killall Dock; sleep 3; [[ -e /Applications/Dashboard.app ]] && open /Applications/Dashboard.app", killDashboardHandler);});
    showStatus(null,'Widget upgraded! Please reload (&#x2318;R) to ensure a clean upgrade.',10,null);
    justShownRestartMessage = true;
}

function killDashboardHandler () {
    return;
}

function makeKey(key) {
    return widget.identifier + "-" + key;
}
